@@ -1,15 +1,18 @@
 import copy
 import numpy
-import pandas
+import csv
 from io import StringIO
+from numbers import Number
 from typing import (
     Any, Callable, Dict, List, Optional,
     Sequence, Tuple, Union, Iterable,
+    TypeVar,
 )
 
 from cosapp.core.numerics.basics import MathematicalProblem, SolverResults
 from cosapp.core.numerics.enum import NonLinearMethods
 from cosapp.core.numerics.root import root
+from cosapp.drivers.driver import Driver
 from cosapp.drivers.abstractsolver import AbstractSolver
 from cosapp.drivers.driver import Driver
 from cosapp.drivers.runsinglecase import RunSingleCase
@@ -20,6 +23,8 @@ from cosapp.utils.logging import LogFormat, LogLevel
 import logging
 logger = logging.getLogger(__name__)
 
+AnyDriver = TypeVar("AnyDriver", bound=Driver)
+
 
 class NonLinearSolver(AbstractSolver):
     """Solve mathematical problem with algebraic variables.
@@ -35,37 +40,40 @@ class NonLinearSolver(AbstractSolver):
     """
 
     __slots__ = (
-        '__method', '__option_aliases', '__trace',
+        '__method', '__option_aliases', '__trace', '__results',
         '__design_unknowns', 'compute_jacobian', 'jac_lup', 'jac',
     )
 
     def __init__(self, 
         name: str, 
-        owner: "Optional[cosapp.systems.System]" = None, 
+        owner: Optional["cosapp.systems.System"] = None, 
         method: Union[NonLinearMethods, str] = NonLinearMethods.NR, 
         **kwargs
     ) -> None:
-        """Initialize a driver
+        """Initialize driver
 
         Parameters
         ----------
         name: str, optional
-            Name of the `Module`
-        owner : System, optional
-            :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
+            Name of the `Driver`.
+        owner: System, optional
+            :py:class:`~cosapp.systems.system.System` to which this driver belong; defaults to `None`.
         method : Union[NonLinearMethods, str]
             Resolution method to use
-        **kwargs : Dict[str, Any]
-            Optional keywords arguments
+        **kwargs:
+            Additional keywords arguments forwarded to base class.
         """
         super().__init__(name, owner, **kwargs)
 
         if isinstance(method, str):
-            method = NonLinearMethods(method)
+            method = NonLinearMethods[method]
+        else:
+            check_arg(method, 'method', NonLinearMethods)
         self.__method = method
         self.__option_aliases = dict()
         self.__set_method(method, **kwargs)
         self.__trace: List[Dict[str, Any]] = list()
+        self.__results: SolverResults = None
 
         self.compute_jacobian = True  # type: bool
             # desc='Should the Jacobian matrix be computed?'
@@ -87,7 +95,14 @@ class NonLinearSolver(AbstractSolver):
         """NonLinearMethods : Selected solver algorithm."""
         return self.__method
 
-    def add_child(self, child: 'Driver', execution_index: Optional[int] = None) -> 'Driver':
+    @property
+    def results(self) -> SolverResults:
+        """SolverResults: structure containing solver results,
+        together with additional detail.
+        """
+        return self.__results
+
+    def add_child(self, child: AnyDriver, execution_index: Optional[int] = None) -> AnyDriver:
         """Add a child `Driver` to the current `Driver`.
 
         When adding a child `Driver`, it is possible to specified its position in the execution order.
@@ -193,49 +208,65 @@ class NonLinearSolver(AbstractSolver):
                 "*" * 40,
             ])
         )
-        self.problem = MathematicalProblem(self.name, self.owner)
         handler = DesignProblemHandler(self.owner)
         handler.design.extend(self._raw_problem, equations=False)
         handler.offdesign.extend(self._raw_problem, unknowns=False)
-        handler.problems = handler.export_problems()  # resolve aliasing
+        handler.prune()  # resolve aliasing
         self.__design_unknowns = handler.design.unknowns
-        self.problem.extend(handler.design)
-        # handler.offdesign.extend(self.owner.get_unsolved_problem(), copy=True)
         self.initial_values = numpy.append(self.initial_values, self.get_init())
         # Set of unknown names to detect design/offdesign conflicts
-        design_unknowns = set(handler.design.unknowns)
-
-        for child in self.children.values():
-            if isinstance(child, RunSingleCase):
-                local = child.processed_problems
-                design_unknowns |= set(local.design.unknowns)
-                common = design_unknowns.intersection(local.offdesign.unknowns)
-                if common:
-                    kind = "unknown"
-                    if len(common) > 1:
-                        names = ", ".join(repr(v) for v in sorted(common))
-                        names = f"({names}) are"
-                        kind += "s"
-                    else:
-                        names = f"{common.pop()!r} is"
-                    raise ValueError(
-                        f"{names} defined as design and off-design {kind} in {child.name!r}"
-                    )
-                self.__design_unknowns.update(local.design.unknowns)
-                # Enforce solver-level off-design problem to child case
-                case_problem = child.add_offdesign_problem(handler.offdesign)
-                self.problem.extend(case_problem, copy=False)
-                self.initial_values = numpy.append(self.initial_values, child.get_init(self.force_init))
+        design_unknown_set = set(handler.design.unknowns)
+        # Create assembled problem
+        problem = self.problem = MathematicalProblem(self.name, self.owner)
+        problem.extend(handler.design)
 
+        run_cases: List[RunSingleCase] = []
+        for driver in self.children.values():
+            if isinstance(driver, RunSingleCase):
+                run_cases.append(driver)
             else:
                 logger.warning(
-                    f"Including Driver {child.name!r} without iteratives in Driver {self.name!r} is not numerically advised."
+                    f"Including Driver {driver.name!r} without iteratives in Driver {self.name!r} is not numerically advised."
                 )
 
+        if len(run_cases) > 1:
+            # If more than one RunSingleCase drivers are present,
+            # use a name wrapper to distinguish dict entries for
+            # residues and off-design unknowns in global problem
+            get_key_wrapper = lambda case: (
+               lambda key: f"{case.name}[{key}]"
+            )
+        else:
+            get_key_wrapper = lambda case: None
+
+        for case in run_cases:
+            local = case.processed_problems
+            design_unknown_set.update(local.design.unknowns)
+            common = design_unknown_set.intersection(local.offdesign.unknowns)
+            if common:
+                kind = "unknown"
+                if len(common) > 1:
+                    names = ", ".join(repr(v) for v in sorted(common))
+                    names = f"({names}) are"
+                    kind += "s"
+                else:
+                    names = f"{common.pop()!r} is"
+                raise ValueError(
+                    f"{names} defined as design and off-design {kind} in {case.name!r}"
+                )
+            self.__design_unknowns.update(local.design.unknowns)
+            # Enforce solver-level off-design problem to child case
+            case.add_offdesign_problem(handler.offdesign)
+            # Assemble case problems (design & off-design)
+            wrapper = get_key_wrapper(case)
+            problem.extend(local.design, copy=False, residue_wrapper=wrapper)
+            problem.extend(local.offdesign, copy=False, unknown_wrapper=wrapper, residue_wrapper=wrapper)
+            self.initial_values = numpy.append(self.initial_values, case.get_init(self.force_init))
+
         logger.debug(
             "\n".join([
                 "Mathematical problem:",
-                f"{'<empty>' if self.problem.shape == (0, 0) else self.problem}",
+                f"{'<empty>' if self.problem.is_empty() else self.problem}",
             ])
         )
 
@@ -256,9 +287,7 @@ class NonLinearSolver(AbstractSolver):
 
         return full_init
 
-    def _fresidues(self,
-        x: Sequence[float],
-    ) -> numpy.ndarray:
+    def _fresidues(self, x: Sequence[float]) -> numpy.ndarray:
         """
         Method used by the solver to take free variables values as input and values of residues as
         output (after running the System).
@@ -280,9 +309,9 @@ class NonLinearSolver(AbstractSolver):
         self.set_iteratives(x)
 
         # Run all points
-        for child in self.exec_order:
-            logger.debug(f"Call {child}.run_once")
-            self.children[child].run_once()
+        for child in self.children.values():
+            logger.debug(f"Call {child.name}.run_once()")
+            child.run_once()
 
         residues = self.problem.residues_vector
         logger.debug(f"Residues: {residues!r}")
@@ -329,12 +358,14 @@ class NonLinearSolver(AbstractSolver):
                 options=self.options,
             )
 
+            self.__results = results
             self.__trace = getattr(results, "trace", list())
 
             if results.success:
                 self.status = ''
                 self.error_code = '0'
                 logger.info(f"solver : {self.name}{results.message}")
+            
             else:
                 self.status = 'ERROR'
                 self.error_code = '9'
@@ -419,37 +450,46 @@ class NonLinearSolver(AbstractSolver):
             emit_record = False
 
             message = ""
-            for i, info in enumerate(self.__trace):
+            unknown_trace = None
+            residue_trace = None
+            residue_names = self.problem.residues_names
+            unknown_names = self.problem.unknowns_names
+            for i, record in enumerate(self.__trace):
                 if i == 0:
-                    unknowns_trace = info["x"]
-                    residues_trace = info["residues"]
+                    unknown_trace = record["x"]
+                    residue_trace = record["residues"]
+                else:
+                    unknown_trace = numpy.vstack((unknown_trace, record["x"]))
+                    residue_trace = numpy.vstack((residue_trace, record["residues"]))
+                try:
+                    jac_record = record["jac"]
+                except KeyError:
+                    continue
                 else:
-                    unknowns_trace = numpy.vstack((unknowns_trace, info["x"]))
-                    residues_trace = numpy.vstack((residues_trace, info["residues"]))
-
-                if "jac" in info:
                     message += f"Iteration {i}\n"
-                    container = StringIO()
-                    numpy.savetxt(container, info["jac"], delimiter=",")
-                    jacobian = container.getvalue()
-                    unknowns = ", ".join(self.problem.unknowns_names)
+                    with StringIO() as container:
+                        numpy.savetxt(container, jac_record, delimiter=",")
+                        jacobian = container.getvalue()
+                    unknowns = ", ".join(unknown_names)
                     message += f"New Jacobian matrix:\n,{unknowns}\n"
-                    for residue, line in zip(self.problem.residues_names, jacobian.splitlines()):
+                    for residue, line in zip(residue_names, jacobian.splitlines()):
                         message += f"{residue}, {line}\n"
 
-            if len(self.__trace) > 0:
-                if len(self.__trace) == 1:
-                    size = self.problem.shape[0]
-                    unknowns_trace = unknowns_trace.reshape((1, size))
-                    residues_trace = residues_trace.reshape((1, size))
-                unknowns_df = pandas.DataFrame(unknowns_trace, columns=self.problem.unknowns_names)
-                container = StringIO()
-                unknowns_df.to_csv(container, line_terminator="\n")
-                message += f"Unknowns\n{container.getvalue()}\n"
-                residues_df = pandas.DataFrame(residues_trace, columns=self.problem.residues_names)
-                container = StringIO()
-                residues_df.to_csv(container, line_terminator="\n")
-                message += f"Residues\n{container.getvalue()}\n"
+            def format_record(records: numpy.ndarray, headers: List[str]) -> str:
+                records = numpy.atleast_2d(records)
+                with StringIO() as stream:
+                    writer = csv.writer(stream, delimiter=",", lineterminator="\n")
+                    writer.writerows([headers, *records])
+                    return stream.getvalue()
+
+            if unknown_trace is not None:
+                message += "Unknowns\n{}\n".format(
+                    format_record(unknown_trace, unknown_names)
+                )
+            if residue_trace is not None:
+                message += "Residues\n{}\n".format(
+                    format_record(residue_trace, residue_names)
+                )
 
             handler.log(
                 LogLevel.FULL_DEBUG,
@@ -466,16 +506,15 @@ class NonLinearSolver(AbstractSolver):
         self.__option_aliases = dict()
 
         if self.__method == NonLinearMethods.NR:
-            self.options.declare('tol', 1.0e-5, dtype=float, allow_none=True,
+            self.options.declare('tol', 'auto', dtype=(float, str), allow_none=True,
                 desc='Absolute tolerance (in max-norm) for the residual.')
             self.options.declare('max_iter', 500, dtype=int,
                 desc='The maximum number of iterations.')
-            self.options.declare('eps', 1.0e-4, dtype=float, allow_none=True,
-                desc='A suitable step length for the forward-difference approximation of the Jacobian (for fprime=None).'
-                    ' If eps is smaller than machine precision u, it is assumed that the relative errors in the'
-                    ' functions are of the order of u.')
-            self.options.declare('factor', 1.0, dtype=float, allow_none=True, lower=1e-3, upper=1.0,
-                desc='A parameter determining the initial step bound factor * norm(diag * x). Should be in the interval [0.1, 1].')
+            # Note: use a power of 2 for `eps`, to guaranty machine-precision accurate gradients in linear problems
+            self.options.declare('eps', 2**(-16), dtype=float, allow_none=True, lower=2**(-30),
+                desc='Relative step length for the forward-difference approximation of the Jacobian.')
+            self.options.declare('factor', 1.0, dtype=Number, allow_none=True, lower=1e-3, upper=1.0,
+                desc='A parameter determining the initial step bound factor * norm(diag * x). Should be in interval [0.001, 1].')
             self.options.declare('partial_jac', True, dtype=bool, allow_none=False,
                 desc='Defines if partial Jacobian updates can be computed before a complete Jacobian matrix update.')
             self.options.declare('partial_jac_tries', 10, dtype=int, allow_none=False, lower=1, upper=10,
@@ -494,13 +533,16 @@ class NonLinearSolver(AbstractSolver):
                 desc='Max relative step for parameters iterated by solver.')
             self.options.declare('history', False, dtype=bool, allow_none=False,
                 desc='Request saving the resolution trace.')
+            self.options.declare('tol_update_period', 4, dtype=int, lower=1, allow_none=False,
+                desc="Tolerance update period, in iteration number, when tol='auto'.")
+            self.options.declare('tol_to_noise_ratio', 16, dtype=Number, lower=1.0, allow_none=False,
+                desc="Tolerance-to-noise ratio, when tol='auto'.")
 
         elif self.__method == NonLinearMethods.POWELL:
             self.__option_aliases = {
                 'tol': 'xtol',
                 'max_eval': 'maxfev',
             }
-
             self.options.declare('xtol', 1.0e-7, dtype=float,
                 desc="The calculation will terminate if the relative error between two consecutive iterations is at most tol.")
             self.options.declare('maxfev', 0, dtype=int,
@@ -521,7 +563,6 @@ class NonLinearSolver(AbstractSolver):
                 'min_rel_step': 'xtol',
                 'min_abs_step': 'xatol',
             }
-
             self.options.declare('nit', 100, dtype=int,
                 desc='Number of iterations to perform. If omitted (default), iterate unit tolerance is met.')
             self.options.declare('maxiter', 200, dtype=int,
