@@ -3,6 +3,7 @@ Ports package of CoSApp.
 """
 from .enum import PortType, Scope, Validity
 from .exceptions import ScopeError
+from .connectors import Connector, ConnectorError
 from .port import Port, ModeVarPort
 from .units import UnitError, add_offset_unit, add_unit, convert_units
 
@@ -10,6 +11,8 @@ __all__ = [
     "ModeVarPort",
     "Port",
     "PortType",
+    "Connector",
+    "ConnectorError",
     "Scope",
     "ScopeError",
     "Validity",
