@@ -1,4 +1,5 @@
 """This module define the basic class encapsulating a variable attributes."""
+from __future__ import annotations
 import abc
 import array
 import numpy
@@ -110,7 +111,7 @@ class BaseVariable(abc.ABC):
         self._scope = scope  # type: Scope
 
     @abc.abstractmethod
-    def copy(self, port: "BasePort", name: Optional[str] = None) -> "BaseVariable":
+    def copy(self, port: "cosapp.ports.port.BasePort", name: Optional[str] = None) -> BaseVariable:
         pass
 
     @abc.abstractmethod
@@ -876,12 +877,16 @@ class Variable(BaseVariable):
             "distribution": self.distribution.__json__() if self.distribution else None,
         })
         for key in ["valid_range", "limits"]:
-            tmp_val = list(getattr(self, key))
-            for idx, val in enumerate(tmp_val):
-                if numpy.isinf(val):
-                    tmp_val[idx] = str(val) 
-            if  tmp_val == ["-inf", "inf"]:
+            try:
+                tmp_val = list(getattr(self, key))
+                for idx, val in enumerate(tmp_val):
+                    if numpy.isinf(val):
+                        tmp_val[idx] = str(val) 
+                if  tmp_val == ["-inf", "inf"]:
+                    tmp_val = None                
+            except TypeError:
                 tmp_val = None
+
             data[key] = tmp_val 
         
         return data
