@@ -34,6 +34,11 @@ def pull_variables(
         CommonPorts.MODEVARS_IN.value: 'add_inward_modevar',
         CommonPorts.MODEVARS_OUT.value: 'add_outward_modevar',
     }
+    def log_debug(parent_attr, child_attr) -> str:
+        logger.debug(
+            f"{parent_attr} has been duplicated from {child_attr}"
+            f" - including validation range and scope."
+        )
     def copy_variable(
         child: System,
         port_name: str,
@@ -47,16 +52,14 @@ def pull_variables(
         lock_status = parent._locked
         parent._locked = False
         # Call `add_inward`, `add_outward`, etc., depending on context
-        method = getattr(parent, method_map[port_name])
-        method(parent_var, value)
+        add_to_parent = getattr(parent, method_map[port_name])
+        add_to_parent(parent_var, value)
         parent._locked = lock_status
         # Copy full variable detail in parent port
         parent_port.copy_variable_from(child_port, child_var, parent_var)
-        logger.debug(
-            "{}.{} has been duplicated from {}.{} - including "
-            "validation range and scope.".format(
-                parent.name, parent_var, child.name, child_var
-            )
+        log_debug(
+            f"{parent.name}.{parent_var}",
+            f"{parent.name}.{child.name}.{child_var}",
         )
 
     if isinstance(pulling, str):
@@ -65,38 +68,37 @@ def pull_variables(
         pulling = dict(zip(pulling, pulling))
 
     for child_attr_name, parent_attr_name in pulling.items():
-        child_port = child[child_attr_name]
+        child_attr = child[child_attr_name]
 
-        if isinstance(child_port, Port):
+        if isinstance(child_attr, Port):
             if parent_attr_name not in parent:
-                pulled_port = child_port.copy(parent_attr_name)
+                pulled_port = child_attr.copy(parent_attr_name)
                 parent._add_port(pulled_port)
-                logger.debug(
-                    f"Port {pulled_port.contextual_name} has been duplicated from"
-                    f" {child_port.contextual_name} - including validation range and scope."
+                log_debug(
+                    f"Port {pulled_port.contextual_name}",
+                    f"{parent.name}.{child_attr.contextual_name}",
                 )
             else:
                 pulled_port = parent[parent_attr_name]
             
-            parent.connect(child_port, pulled_port)
+            parent.connect(child_attr, pulled_port)
 
         else:  # ExtensiblePort (inwards or outwards)
-            if isinstance(child_port, BasePort):  # Pulling all inwards or outwards
-                port_name = child_port.name
+            if isinstance(child_attr, BasePort):  # Pulling all inwards or outwards
+                port_name = child_attr.name
                 parent_port = parent[port_name]
 
-                for varname in child_port:
+                for varname, value in child_attr.items():
                     if varname not in parent_port:
-                        value = child_port[varname]
                         copy_variable(child, port_name, varname, varname, value)
 
-                parent.connect(child_port, parent_port, list(child_port))
+                parent.connect(child_attr, parent_port, list(child_attr))
 
             else:  # Pulling individual variable
                 var = child.name2variable[child_attr_name]
                 port_name = var.mapping.name
 
                 if parent_attr_name not in parent[port_name]:
-                    copy_variable(child, port_name, child_attr_name, parent_attr_name, child_port)
+                    copy_variable(child, port_name, child_attr_name, parent_attr_name, child_attr)
 
                 parent.connect(child[port_name], parent[port_name], {child_attr_name: parent_attr_name})
